import { defineStore } from 'pinia'
import { ref, computed } from 'vue'
import { Room, RoomRole, RoomSettings } from '@/types/room'
import Taro from '@tarojs/taro'

// 定义云函数返回结果的接口
interface RoomResult {
  room?: Room;
  roomCode?: string;
  role?: RoomRole;
  error?: string;
  success?: boolean;
  [key: string]: any;
}

export const useRoomStore = defineStore('room', () => {
  // 当前房间
  const currentRoom = ref<Room | null>(null)
  // 用户在当前房间的角色
  const userRole = ref<RoomRole | null>(null)
  // 是否正在加载
  const loading = ref(false)
  
  // 用户是否是主持人
  const isHost = computed(() => userRole.value === RoomRole.HOST)
  // 用户是否是玩家
  const isPlayer = computed(() => userRole.value === RoomRole.PLAYER)
  // 房间是否已满
  const isRoomFull = computed(() => {
    if (!currentRoom.value) return false
    return currentRoom.value.players.length >= currentRoom.value.settings.maxPlayers
  })
  
  // 创建房间
  async function createRoom(gameType: string, settings: RoomSettings) {
    loading.value = true
    try {
      const res = await Taro.cloud.callFunction({
        name: 'createRoom',
        data: { gameType, settings }
      })
      
      // 使用类型断言
      const result = res.result as RoomResult
      
      if (result && result.room) {
        currentRoom.value = result.room
        userRole.value = RoomRole.HOST
        return { success: true, roomCode: currentRoom.value.roomCode }
      }
      return { success: false, error: '创建房间失败' }
    } catch (error) {
      console.error('创建房间失败', error)
      return { success: false, error: '创建房间失败' }
    } finally {
      loading.value = false
    }
  }
  
  // 加入房间
  async function joinRoom(roomCode: string) {
    loading.value = true
    try {
      const res = await Taro.cloud.callFunction({
        name: 'joinRoom',
        data: { roomCode }
      })
      
      // 使用类型断言
      const result = res.result as RoomResult
      
      if (result && result.room) {
        currentRoom.value = result.room
        userRole.value = RoomRole.PLAYER
        return { success: true, room: currentRoom.value }
      }
      return { success: false, error: result?.error || '加入房间失败' }
    } catch (error) {
      console.error('加入房间失败', error)
      return { success: false, error: '加入房间失败' }
    } finally {
      loading.value = false
    }
  }
  
  // 获取房间详情
  async function getRoomDetail(roomId: string) {
    loading.value = true
    try {
      const res = await Taro.cloud.callFunction({
        name: 'getRoomDetail',
        data: { roomId }
      })
      
      // 使用类型断言
      const result = res.result as RoomResult
      
      if (result && result.room) {
        currentRoom.value = result.room
        // 这里可能需要从结果中获取用户角色
        if (result.role) {
          userRole.value = result.role
        }
        return { success: true, room: currentRoom.value }
      }
      return { success: false, error: '获取房间详情失败' }
    } catch (error) {
      console.error('获取房间详情失败', error)
      return { success: false, error: '获取房间详情失败' }
    } finally {
      loading.value = false
    }
  }
  
  // 离开房间
  async function leaveRoom() {
    if (!currentRoom.value) return { success: true }
    
    try {
      const res = await Taro.cloud.callFunction({
        name: 'leaveRoom',
        data: { roomId: currentRoom.value.id }
      })
      
      // 使用类型断言
      const result = res.result as RoomResult
      
      if (result && result.success) {
        currentRoom.value = null
        userRole.value = null
        return { success: true }
      }
      return { success: false, error: '离开房间失败' }
    } catch (error) {
      console.error('离开房间失败', error)
      return { success: false, error: '离开房间失败' }
    }
  }
  
  // 开始游戏
  async function startGame() {
    if (!currentRoom.value || !isHost.value) {
      return { success: false, error: '没有权限开始游戏' }
    }
    
    try {
      const res = await Taro.cloud.callFunction({
        name: 'startGame',
        data: { roomId: currentRoom.value.id }
      })
      
      // 使用类型断言
      const result = res.result as { gameId?: string; success?: boolean; error?: string }
      
      if (result && result.gameId) {
        return { success: true, gameId: result.gameId }
      }
      return { success: false, error: result?.error || '开始游戏失败' }
    } catch (error) {
      console.error('开始游戏失败', error)
      return { success: false, error: '开始游戏失败' }
    }
  }
  
  // 清除房间数据
  function clearRoom() {
    currentRoom.value = null
    userRole.value = null
  }
  
  return {
    currentRoom,
    userRole,
    loading,
    isHost,
    isPlayer,
    isRoomFull,
    createRoom,
    joinRoom,
    getRoomDetail,
    leaveRoom,
    startGame,
    clearRoom
  }
})