import { defineStore } from 'pinia'
import { ref } from 'vue'
import type { User, UserSettings } from '@/types/user'
import Taro from '@tarojs/taro'

// 定义云函数返回结果的接口
interface LoginResult {
  user?: User;
  openid?: string;
  [key: string]: any;
}

export const useUserStore = defineStore('user', () => {
  // 用户信息
  const userInfo = ref<User | null>(null)
  // 用户设置
  const settings = ref<UserSettings>({
    theme: 'light',
    notificationsEnabled: true,
    vibrationEnabled: true
  })
  // 是否已登录
  const isLoggedIn = ref(false)
  
  // 登录
  async function login() {
    try {
      // 微信登录
      const { code } = await Taro.login()
      
      // 调用云函数进行登录
      const res = await Taro.cloud.callFunction({
        name: 'login',
        data: { code }
      })
      
      // 使用类型断言来明确结果类型
      const result = res.result as LoginResult
      
      if (result && result.user) {
        userInfo.value = result.user
        isLoggedIn.value = true
        return true
      }
      return false
    } catch (error) {
      console.error('登录失败', error)
      return false
    }
  }
  
  // 获取用户信息
  async function getUserInfo() {
    try {
      const res = await Taro.cloud.callFunction({
        name: 'getUserInfo'
      })
      
      // 使用类型断言
      const result = res.result as LoginResult
      
      if (result && result.user) {
        userInfo.value = result.user
        return userInfo.value
      }
      return null
    } catch (error) {
      console.error('获取用户信息失败', error)
      return null
    }
  }
  
  // 更新用户设置
  function updateSettings(newSettings: Partial<UserSettings>) {
    settings.value = { ...settings.value, ...newSettings }
    // 可以在这里同步到服务端
  }
  
  // 退出登录
  function logout() {
    userInfo.value = null
    isLoggedIn.value = false
    // 可以在这里清除存储的登录状态
  }
  
  return {
    userInfo,
    settings,
    isLoggedIn,
    login,
    getUserInfo,
    updateSettings,
    logout
  }
})